"""Register django_secretary to django admin."""
from typing import Dict, Optional, Union, cast

from django import forms
from django.contrib.admin import ModelAdmin, TabularInline, site
from django.core.files.base import ContentFile
from django.db import models
from django.db.models.fields.files import FieldFile

from .models import Asset, Tag, Template, TemplateCollection


class TextFileField(forms.CharField):
    """Form field which renders file content as a text."""

    widget = forms.Textarea

    def __init__(self, *, strip: bool = False, **kwargs):
        # Disable stripping by default and ignore any provided max_length.
        kwargs.pop('max_length', None)
        super().__init__(strip=strip, **kwargs)

    def prepare_value(self, value: Optional[Union[str, FieldFile]]) -> Optional[str]:
        """Prepare value for rendering in widget."""
        # Model provdes `FieldFile`, form `str`
        if hasattr(value, 'read'):
            return cast(FieldFile, value).read().decode()
        return value

    def to_python(self, data: Optional[str]) -> Optional[ContentFile]:
        """Convert value from widget to python."""
        data = super().to_python(data)
        if not data:
            return None
        # models.FileField requires file to have a name.
        return ContentFile(data, name='dummy.txt')

    def has_changed(self, initial: Optional[FieldFile], data: Optional[str]) -> bool:
        """Return True if data differs from initial."""
        if self.disabled:
            return False
        initial_content = self.prepare_value(initial)
        data_content = data or None
        return initial_content != data_content


class TemplateForm(forms.ModelForm):
    """Form for a template model."""

    def clean(self) -> Dict:
        """Modify cleaned data to prevent creation of a new file if the content haven't changed."""
        cleaned_data = self.cleaned_data
        # If the file haven't changed, set its value to None to signal models.FileField it haven't changed.
        if 'content' not in self.changed_data:
            cleaned_data['content'] = None
        return cleaned_data


class AssetInline(TabularInline):
    """Inline admin for assets."""

    model = Asset
    formfield_overrides = {
        models.TextField: {'widget': forms.TextInput},
    }


class TagAdmin(ModelAdmin):
    """Model admin for tags."""

    list_display = ('name', )
    formfield_overrides = {
        models.TextField: {'widget': forms.TextInput},
    }


class TemplateAdmin(ModelAdmin):
    """Model admin for templates."""

    list_display = ('name', 'is_active', 'uuid')
    list_filter = ('is_active', )
    inlines = (AssetInline, )
    form = TemplateForm
    formfield_overrides = {
        models.TextField: {'widget': forms.TextInput},
        models.FileField: {'form_class': TextFileField, 'widget': forms.Textarea(attrs={'cols': 80, 'rows': 15})},
    }


class TemplateCollectionAdmin(ModelAdmin):
    """Model admin for template collections."""

    list_display = ('name', )
    filter_horizontal = ('tags', )
    formfield_overrides = {
        models.TextField: {'widget': forms.TextInput},
    }


site.register(Tag, TagAdmin)
site.register(Template, TemplateAdmin)
site.register(TemplateCollection, TemplateCollectionAdmin)
