"""Renderers for django-secretary."""
from typing import Any, Dict

import weasyprint
from django.template import loader
from rest_framework.renderers import BaseRenderer

from .utils.fetchers import FileFetcher


class TextRenderer(BaseRenderer):
    """Renderer which renders secretary template as a plain text.

    Attributes:
        exception_template: A template for rendering an error.
    """

    media_type = 'text/plain'
    format = 'txt'

    exception_template = 'django_secretary/render_error.txt'

    def render(self, data: Any, media_type: str = None, renderer_context: Dict[str, Any] = None) -> str:
        """Render secretary template as a plain text."""
        renderer_context = renderer_context or {}
        if renderer_context['response'].exception:
            return loader.render_to_string(self.exception_template, {'errors': data})
        view = renderer_context['view']
        template = view.get_object()
        return template.render(data)


class HtmlRenderer(TextRenderer):
    """Renderer which renders secretary template as HTML."""

    media_type = 'text/html'
    format = 'html'

    exception_template = 'django_secretary/render_error.html'


class PdfRenderer(TextRenderer):
    """Renderer which renders secretary template as PDF."""

    media_type = 'application/pdf'
    format = 'pdf'

    def render(self, data: Any, media_type: str = None, renderer_context: Dict[str, Any] = None) -> str:
        """Render secretary template as PDF."""
        rendered_template = super().render(data, media_type=media_type, renderer_context=renderer_context)
        renderer_context = renderer_context or {}
        request = renderer_context['request']
        fetcher = FileFetcher(request.build_absolute_uri())
        pdf_renderer = weasyprint.HTML(string=rendered_template, url_fetcher=fetcher.fetch,
                                       base_url=request.build_absolute_uri())
        return pdf_renderer.write_pdf()
