from typing import cast
from unittest.mock import sentinel

from django.core.files.base import ContentFile
from django.test import SimpleTestCase

from django_secretary.admin import TemplateForm, TextFileField
from django_secretary.models import Template


class TextFileFieldTest(SimpleTestCase):
    def test_strip_disabled(self):
        field = TextFileField()
        self.assertFalse(field.strip)

    def test_max_length_ignored(self):
        field = TextFileField(max_length=10)
        # Check it doesn't raise error
        value = 'Really long value'
        result = field.clean(value)
        self.assertEqual(result.read(), value)

    def test_prepare_value(self):
        field = TextFileField()
        test_data = (
            # value, result
            (None, None),
            ('Gazpacho!', 'Gazpacho!'),
            (ContentFile(b'Gazpacho!'), 'Gazpacho!'),
        )
        for value, result in test_data:
            with self.subTest(value=value, result=result):
                self.assertEqual(field.prepare_value(value), result)

    def test_to_python_empty(self):
        field = TextFileField()
        test_data = (
            # data, result
            (None, None),
            ('', None),
        )
        for data, result in test_data:
            with self.subTest(data=data, result=result):
                self.assertEqual(field.to_python(data), result)

    def test_to_python(self):
        field = TextFileField()
        result = field.to_python('Gazpacho!')

        result = cast(ContentFile, result)
        self.assertEqual(result.read(), 'Gazpacho!')
        self.assertEqual(result.name, 'dummy.txt')

    def test_has_changed(self):
        field = TextFileField()
        test_data = (
            # initial, data, result
            (None, None, False),
            (None, '', False),
            (ContentFile(b'Gazpacho!'), 'Gazpacho!', False),
            (None, 'Gazpacho!', True),
            (ContentFile(b'Gazpacho!'), 'Quagaars!', True),
            (ContentFile(b'Gazpacho!'), '', True),
        )
        for initial, data, result in test_data:
            with self.subTest(initial=initial, data=data, result=result):
                self.assertEqual(field.has_changed(initial, data), result)

    def test_has_changed_disabled(self):
        field = TextFileField(disabled=True)
        self.assertFalse(field.has_changed(sentinel.initial, sentinel.data))


class TestTemplateForm(TemplateForm):
    """Template form for testing."""
    content = TextFileField()

    class Meta:
        model = Template
        fields = ('content', )


class TemplateFormTest(SimpleTestCase):
    def test_clean_new(self):
        form = TestTemplateForm({'content': 'Gazpacho!'})
        form.is_valid()
        self.assertEqual(form.cleaned_data['content'].read(), 'Gazpacho!')

    def test_clean_no_change(self):
        instance = Template(content=ContentFile(b'Gazpacho!', name='dummy.txt'))
        form = TestTemplateForm({'content': 'Gazpacho!'}, instance=instance)
        form.is_valid()
        self.assertIsNone(form.cleaned_data['content'])

    def test_clean_change(self):
        instance = Template(content=ContentFile(b'Gazpacho!', name='dummy.txt'))
        form = TestTemplateForm({'content': 'Quagaars!'}, instance=instance)
        form.is_valid()
        self.assertEqual(form.cleaned_data['content'].read(), 'Quagaars!')
