from functools import partial
from typing import Any, Dict
from unittest.mock import sentinel
from uuid import UUID

from django.forms import Media
from django.template import TemplateDoesNotExist, TemplateSyntaxError
from django.test import SimpleTestCase, TestCase
from django.test.client import store_rendered_templates
from django.test.signals import template_rendered
from testfixtures import TempDirectory

from django_secretary.constants import MimeType
from django_secretary.models import Asset, Tag, Template, make_file_name

from .utils import TEMPLATES, create_asset, create_template


class TestMakeFileName(SimpleTestCase):
    def test_template(self):
        template = Template(name='example.html')
        self.assertEqual(make_file_name(template, sentinel.original), 'example-{}.html'.format(template.uuid))

    def test_asset(self):
        asset = Asset(name='example.css')
        self.assertEqual(make_file_name(asset, sentinel.original), 'example-{}.css'.format(asset.uuid))

    def test_prefix(self):
        template = Template(name='example.html')
        self.assertEqual(make_file_name(template, sentinel.original, 'pre/fix'),
                         'pre/fix/example-{}.html'.format(template.uuid))


class TestTemplate(TestCase):
    def setUp(self):
        # Use temporary file storage
        tmp_dir = TempDirectory()
        self.addCleanup(tmp_dir.cleanup)
        storage_patcher = self.settings(MEDIA_ROOT=tmp_dir.path)
        self.addCleanup(storage_patcher.disable)
        storage_patcher.enable()

        # Capture template rendering arguments
        self.template_rendered = {}  # type: Dict[str, Any]
        dispatch_uid = 'test-template-render-{}'.format(id(self))
        template_rendered.connect(partial(store_rendered_templates, self.template_rendered), dispatch_uid=dispatch_uid,
                                  weak=False)
        self.addCleanup(template_rendered.disconnect, dispatch_uid=dispatch_uid)

    def test_str(self):
        template = Template(name='example.html', uuid=UUID(int=42))
        self.assertEqual(str(template), 'example.html (00000000-0000-0000-0000-00000000002a)')

    def test_render(self):
        # Test rendering simple template.
        template = create_template(name='example.html', content="Hello, I'm {{ name }}.")
        context = {'name': 'Kryten'}
        with self.settings(SECRETARY_TEMPLATES=TEMPLATES):
            self.assertEqual(template.render(context), "Hello, I'm Kryten.")

        self.assertEqual(self.template_rendered['templates'][0].name, 'example.html')
        self.assertEqual(self.template_rendered['context'][0]['name'], 'Kryten')
        self.assertEqual(self.template_rendered['context'][0]['template'], template)

    def test_render_parent(self):
        # Test rendering the template with parent template.
        parent = create_template(name='base.html', content="Hello, {% block content %}{% endblock %}")
        template = create_template(
            name='example.html', parent=parent,
            content="{% extends template.parent.name %}{% block content %}I'm {{ name }}.{% endblock %}")
        context = {'name': 'Kryten'}
        with self.settings(SECRETARY_TEMPLATES=TEMPLATES):
            self.assertEqual(template.render(context), "Hello, I'm Kryten.")

        self.assertEqual(self.template_rendered['templates'][0].name, 'example.html')
        self.assertEqual(self.template_rendered['context'][0]['name'], 'Kryten')
        self.assertEqual(self.template_rendered['context'][0]['template'], template)
        self.assertEqual(self.template_rendered['templates'][1].name, 'base.html')
        self.assertEqual(self.template_rendered['context'][1]['name'], 'Kryten')

    def test_render_no_engines(self):
        template = Template()

        with self.settings(SECRETARY_TEMPLATES=[]):
            with self.assertRaisesRegex(TemplateDoesNotExist, "Template .* rendering failed.") as catcher:
                template.render({})

        # No chain reported in this case.
        self.assertEqual(catcher.exception.chain, [])

    def test_render_misconfigured(self):
        # Test error in template engine configuration - no loaders.
        template = Template(name='example.html')
        templates = [{'BACKEND': 'django.template.backends.django.DjangoTemplates', 'OPTIONS': {'loaders': []}}]

        with self.settings(SECRETARY_TEMPLATES=templates):
            with self.assertRaisesRegex(TemplateDoesNotExist, "Template .* rendering failed.") as catcher:
                template.render({})

        self.assertEqual(len(catcher.exception.chain), 1)
        self.assertIsInstance(catcher.exception.chain[0], TemplateDoesNotExist)
        self.assertEqual(catcher.exception.chain[0].args, ('example.html', ))

    def test_render_invalid(self):
        # Test rendering invalid template.
        template = create_template(name='example.html', content="Hello, I'm {% invalid %}")
        with self.settings(SECRETARY_TEMPLATES=TEMPLATES):
            with self.assertRaisesMessage(TemplateSyntaxError, "Invalid block tag on line 1: 'invalid'."):
                template.render({})

    def test_media_empty(self):
        template = Template.objects.create()
        self.assertIsInstance(template.media, Media)
        self.assertEqual(str(template.media), '')

    def test_media(self):
        template = Template.objects.create()
        css = create_asset(template=template, name='example.css', mime_type=MimeType.CSS, content='/* Placeholder */')
        css_url = '/media/assets/example-{}.css'.format(css.uuid)
        with self.settings(MEDIA_URL='/media/'):
            self.assertIsInstance(template.media, Media)
            self.assertHTMLEqual(str(template.media),
                                 '<link href="{}" type="text/css" media="all" rel="stylesheet">'.format(css_url))

    def test_media_no_images(self):
        # Test that media doesn't contain images.
        template = Template.objects.create()
        create_asset(template=template, name='example.svg', mime_type=MimeType.SVG, content='/* Placeholder */')
        self.assertIsInstance(template.media, Media)
        self.assertEqual(str(template.media), '')

    def test_images_empty(self):
        template = Template.objects.create()
        self.assertEqual(template.images, {})

    def test_images(self):
        template = Template.objects.create()
        image = create_asset(template=template, name='example.svg', mime_type=MimeType.SVG, content='/* Placeholder */')
        self.assertEqual(template.images, {'example.svg': image.content})


class TagTest(SimpleTestCase):
    def test_str(self):
        tag = Tag(name='example')
        self.assertEqual(str(tag), 'example')
