import types
from unittest.mock import call, patch, sentinel
from uuid import UUID

from django.test import RequestFactory, TestCase, override_settings
from rest_framework.request import Request
from rest_framework.response import Response
from testfixtures import Comparison

from django_secretary.renderers import PdfRenderer, TextRenderer
from django_secretary.views import TemplateViewSet

from .utils import TEMPLATES, RenderTestMixin, create_template


@override_settings(SECRETARY_TEMPLATES=TEMPLATES)
class TestTextRenderer(RenderTestMixin, TestCase):
    def test_render(self):
        uuid = UUID(int=42)
        create_template(name='example.txt', content="Hello, I'm {{ name }}.", uuid=uuid)
        data = {'name': 'Kryten'}
        request = Request(RequestFactory().get('/dummy/'))
        view = TemplateViewSet(request=request, kwargs={'uuid': str(uuid)})
        renderer_context = {
            'response': Response(),
            'view': view,
        }
        renderer = TextRenderer()

        result = renderer.render(data, renderer_context=renderer_context)

        self.assertEqual(result, "Hello, I'm Kryten.")
        self.assertEqual(self.template_rendered['templates'][0].name, 'example.txt')
        self.assertEqual(self.template_rendered['context'][0]['name'], 'Kryten')

    def test_render_error(self):
        renderer_context = {
            'response': Response(status=500, exception=True),
        }
        renderer = TextRenderer()
        data = {'field_42': ['Invalid value']}

        result = renderer.render(data, renderer_context=renderer_context)

        self.assertEqual(result.strip(), str(data))
        self.assertEqual(self.template_rendered['templates'][0].name, 'django_secretary/render_error.txt')
        self.assertEqual(self.template_rendered['context'][0]['errors'], data)


@override_settings(SECRETARY_TEMPLATES=TEMPLATES)
class TestPdfRenderer(RenderTestMixin, TestCase):
    def test_render(self):
        uuid = UUID(int=42)
        create_template(name='example.html', content="Hello, I'm {{ name }}.", uuid=uuid)
        data = {'name': 'Kryten'}
        request = Request(RequestFactory().get('/dummy/'))
        view = TemplateViewSet(request=request, kwargs={'uuid': str(uuid)})
        renderer_context = {
            'request': request,
            'response': Response(),
            'view': view,
        }
        renderer = PdfRenderer()

        with patch('django_secretary.renderers.weasyprint.HTML') as weasy_mock:
            weasy_mock.return_value.write_pdf.return_value = sentinel.pdf
            result = renderer.render(data, renderer_context=renderer_context)

        self.assertEqual(result, sentinel.pdf)
        self.assertEqual(self.template_rendered['templates'][0].name, 'example.html')
        self.assertEqual(self.template_rendered['context'][0]['name'], 'Kryten')
        calls = [call(string="Hello, I'm Kryten.", url_fetcher=Comparison(types.MethodType),
                      base_url='http://testserver/dummy/'),
                 call().write_pdf()]
        self.assertEqual(weasy_mock.mock_calls, calls)
