from unittest import skipIf

import django
from django.core.exceptions import ImproperlyConfigured
from django.test import SimpleTestCase
from testfixtures import TempDirectory

from django_secretary.utils.fetchers import FileFetcher


class TestFileFetcher(SimpleTestCase):
    test_file_content = b'The test file content.'

    def setUp(self):
        # Set up temporary directory as a storage
        self.tmp_dir = TempDirectory()
        self.addCleanup(self.tmp_dir.cleanup)
        self.tmp_dir.write('test.css', self.test_file_content)

    def test_init(self):
        with self.settings(MEDIA_ROOT=self.tmp_dir.path, MEDIA_URL='/media/'):
            fetcher = FileFetcher('http://example.net/')

        self.assertEqual(fetcher.media_url, 'http://example.net/media/')

    @skipIf(django.VERSION >= (3, 1), "Only affects Django <=3.0")
    def test_init_no_url(self):
        with self.assertRaisesMessage(ImproperlyConfigured, 'Django-secretary fetcher requires non-empty MEDIA_URL.'):
            with self.settings(MEDIA_ROOT=self.tmp_dir.path, MEDIA_URL=''):
                FileFetcher('http://example.net/')

    def test_init_no_root(self):
        with self.assertRaisesMessage(ImproperlyConfigured, 'Django-secretary fetcher requires non-empty MEDIA_ROOT.'):
            with self.settings(MEDIA_ROOT='', MEDIA_URL='/media/'):
                FileFetcher('http://example.net/')

    def test_fetch(self):
        with self.settings(MEDIA_ROOT=self.tmp_dir.path, MEDIA_URL='/media/'):
            fetcher = FileFetcher('http://example.net/')
            data = fetcher.fetch('http://example.net/media/test.css')
            self.addCleanup(data['file_obj'].close)

        self.assertCountEqual(data.keys(), ('file_obj', 'mime_type'))
        self.assertEqual(data['file_obj'].read(), self.test_file_content)
        self.assertEqual(data['mime_type'], 'text/css')

    def test_fetch_not_found(self):
        url = 'http://example.net/media/unknown.txt'
        filename = self.tmp_dir.getpath('unknown.txt')
        with self.assertRaisesMessage(ValueError, 'Media file {} not found at {}.'.format(url, filename)):
            with self.settings(MEDIA_ROOT=self.tmp_dir.path, MEDIA_URL='/media/'):
                fetcher = FileFetcher('http://example.net/')
                fetcher.fetch(url)

    def test_fetch_external(self):
        with self.assertRaisesMessage(ValueError, 'URI http://example.net/other.txt is out of base URL.'):
            with self.settings(MEDIA_ROOT=self.tmp_dir.path, MEDIA_URL='/media/'):
                fetcher = FileFetcher('http://example.net/')
                fetcher.fetch('http://example.net/other.txt')
