from threading import local
from unittest.mock import patch, sentinel

from django.template import Engine, Origin, TemplateDoesNotExist
from django.test import TestCase
from testfixtures import TempDirectory

from django_secretary.models import Template
from django_secretary.utils.templates import TemplateLoader, get_active_template, override_template

from .utils import create_template


class TestGetActiveTemplate(TestCase):
    def setUp(self):
        patcher = patch('django_secretary.utils.templates._ACTIVE_TEMPLATES', local())
        self.addCleanup(patcher.stop)
        self._active_templates = patcher.start()

    def test_no_active(self):
        # Test no active template
        template = Template.objects.create(name='example.html', is_active=True)
        self.assertEqual(get_active_template('example.html'), template)

    def test_no_active_not_found(self):
        # Test no active template
        with self.assertRaisesMessage(TemplateDoesNotExist, "'unknown.html' is neither active, nor found in database."):
            get_active_template('unknown.html')

    def test_active(self):
        self._active_templates.templates = {'example.html': sentinel.template}
        self.assertEqual(get_active_template('example.html'), sentinel.template)

    def test_active_unknown(self):
        self._active_templates.templates = {'example.html': sentinel.template}
        with self.assertRaisesMessage(TemplateDoesNotExist, "'unknown.html' has no active template."):
            get_active_template('unknown.html')


class TestOverrideTemplate(TestCase):
    def setUp(self):
        patcher = patch('django_secretary.utils.templates._ACTIVE_TEMPLATES', local())
        self.addCleanup(patcher.stop)
        self._active_templates = patcher.start()

    def test_override(self):
        # Test override correctly changes template there and back
        template = Template(name='example.html')
        with override_template(template):
            overriden = get_active_template('example.html')

        self.assertEqual(overriden, template)
        # Check template is deactivated when override exited.
        self.assertRaises(TemplateDoesNotExist, get_active_template, 'example.html')

    def test_override_raise(self):
        # Test override correctly changes template back even if error occurs
        template = Template(name='example.html')
        with self.assertRaisesMessage(Exception, "ERROR_IN_OVERRIDE"):
            with override_template(template):
                raise Exception("ERROR_IN_OVERRIDE")

        # Check template is deactivated when override exited.
        self.assertRaises(TemplateDoesNotExist, get_active_template, 'example.html')

    def test_override_nested(self):
        # Test override correctly changes template there and back, even when nested
        template = Template(name='example.html')
        nested_template = Template(name='example.html')
        with override_template(template):
            overriden = get_active_template('example.html')
            with override_template(nested_template):
                overriden_nested = get_active_template('example.html')
            returned = get_active_template('example.html')

        self.assertEqual(overriden, template)
        self.assertEqual(overriden_nested, nested_template)
        self.assertEqual(returned, template)
        # Check template is deactivated when override exited.
        self.assertRaises(TemplateDoesNotExist, get_active_template, 'example.html')

    def test_override_parent(self):
        # Test override correctly changes also parent template
        parent = Template(name='parent.html')
        template = Template(name='example.html', parent=parent)
        with override_template(template):
            overriden = get_active_template('example.html')
            overriden_parent = get_active_template('parent.html')

        self.assertEqual(overriden, template)
        self.assertEqual(overriden_parent, parent)
        # Check templates are deactivated when override exited.
        self.assertRaises(TemplateDoesNotExist, get_active_template, 'example.html')
        self.assertRaises(TemplateDoesNotExist, get_active_template, 'parent.html')


class TestTemplateLoader(TestCase):
    engine = Engine()

    def setUp(self):
        patcher = patch('django_secretary.utils.templates._ACTIVE_TEMPLATES', local())
        self.addCleanup(patcher.stop)
        self._active_templates = patcher.start()

        # Use temporary file storage
        tmp_dir = TempDirectory()
        self.addCleanup(tmp_dir.cleanup)
        storage_patcher = self.settings(MEDIA_ROOT=tmp_dir.path)
        self.addCleanup(storage_patcher.disable)
        storage_patcher.enable()
        self.template = create_template(name='example.html', is_active=True, content='Something completely different')

    def test_get_template_sources(self):
        loader = TemplateLoader(self.engine)
        origin = Origin(name=sentinel.name, loader=loader)
        self.assertCountEqual(loader.get_template_sources(sentinel.name), (origin, ))

    def test_get_contents(self):
        loader = TemplateLoader(self.engine)
        origin = Origin(name='example.html')
        self.assertEqual(loader.get_contents(origin), 'Something completely different')

    def test_get_contents_active(self):
        template = create_template(name='active.html', content='Something else')
        loader = TemplateLoader(self.engine)
        origin = Origin(name='active.html')
        with override_template(template):
            self.assertEqual(loader.get_contents(origin), 'Something else')

    def test_get_contents_not_found(self):
        loader = TemplateLoader(self.engine)
        origin = Origin(name='unknown.html')
        self.assertRaises(TemplateDoesNotExist, loader.get_contents, origin)
