from functools import partial
from typing import Any, Dict

from django.core.files.base import ContentFile
from django.test import Client, TestCase
from django.test.client import store_rendered_templates
from django.test.signals import template_rendered
from testfixtures import TempDirectory

from django_secretary.models import Asset, Template

TEMPLATES = [{'BACKEND': 'django.template.backends.django.DjangoTemplates',
              'OPTIONS': {'loaders': ['django_secretary.utils.templates.TemplateLoader']}}]


class JsonClient(Client):
    """Test client which sends tries to encode all data as JSON."""

    def get(self, path, data=None, follow=False, secure=False, content_type='application/json', **extra):
        """Request a response from the server using GET."""
        data = self._encode_json({} if data is None else data, content_type)
        body = self._encode_data(data, content_type)
        response = self.generic('GET', path, body, content_type, secure=secure, **extra)
        if follow:  # pragma: no cover
            response = self._handle_redirects(response, data=data, **extra)
        return response


def create_template(*, content: str, **kwargs) -> Template:
    """Create Template model instance with defined content."""
    template = Template.objects.create(**kwargs)
    template.content.save('', ContentFile(content))
    template.save()
    return template


def create_asset(*, content: str, **kwargs) -> Asset:
    """Create Asset model instance with defined content."""
    asset = Asset.objects.create(**kwargs)
    asset.content.save('', ContentFile(content))
    asset.save()
    return asset


class RenderTestMixin:
    """Test mixin for tests of template rendering."""

    def setUp(self: TestCase):
        # Use temporary file storage
        tmp_dir = TempDirectory()
        self.addCleanup(tmp_dir.cleanup)
        media_patcher = self.settings(MEDIA_ROOT=tmp_dir.path, MEDIA_URL='/media/')
        self.addCleanup(media_patcher.disable)
        media_patcher.enable()

        # Capture template rendering arguments
        self.template_rendered = {}  # type: Dict[str, Any]
        dispatch_uid = 'test-template-render-{}'.format(id(self))
        template_rendered.connect(partial(store_rendered_templates, self.template_rendered), dispatch_uid=dispatch_uid,
                                  weak=False)
        self.addCleanup(template_rendered.disconnect, dispatch_uid=dispatch_uid)
