"""Custom fetchers for weasyprint."""
import mimetypes
import os
import ssl
from typing import Any, Dict
from urllib.parse import urljoin, urlsplit

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.files.storage import default_storage


class FileFetcher:
    """Fetcher which only fetches files from file system.

    Attributes:
        media_url: Base URL of the media files. All served media files must belong to it.
    """

    def __init__(self, base_url: str):
        """Initialize fetcher."""
        if not settings.MEDIA_URL:
            raise ImproperlyConfigured('Django-secretary fetcher requires non-empty MEDIA_URL.')
        if not settings.MEDIA_ROOT:
            raise ImproperlyConfigured('Django-secretary fetcher requires non-empty MEDIA_ROOT.')
        self.media_url = urljoin(base_url, settings.MEDIA_URL)

    def fetch(self, url: str, timeout: int = 10, ssl_context: ssl.SSLContext = None) -> Dict[str, Any]:
        """Return file based on URL."""
        split_url = urlsplit(url)
        split_media_url = urlsplit(self.media_url)
        if (split_media_url.scheme == split_url.scheme and split_media_url.netloc == split_url.netloc
                and split_url.path.startswith(split_media_url.path)):
            filename = os.path.join(settings.MEDIA_ROOT, *split_url.path[len(split_media_url.path):].split('/'))
            try:
                data = default_storage.open(filename)
            except FileNotFoundError:
                raise ValueError('Media file {} not found at {}.'.format(url, filename))
            else:
                return {'file_obj': data, 'mime_type': mimetypes.guess_type(url)[0]}
        raise ValueError('URI {} is out of base URL.'.format(url))
