"""Template utilities for django-secretary."""
from contextlib import ContextDecorator
from threading import local

from django.template import EngineHandler, Origin, TemplateDoesNotExist
from django.template.loaders.base import Loader

from ..models import Template
from ..settings import SECRETARY_SETTINGS

TEMPLATE_ENGINES = EngineHandler(SECRETARY_SETTINGS.TEMPLATES)

# Thread-safe storage of active templates.
# If the attribute `templates` exists, it contains mapping of template name to `Template` model instance.
_ACTIVE_TEMPLATES = local()


def get_active_template(name: str) -> Template:
    """Return active template or raise an exception.

    Raises:
        TemplateDoesNotExist: If template was not found.
    """
    if hasattr(_ACTIVE_TEMPLATES, 'templates'):
        if name in _ACTIVE_TEMPLATES.templates:
            return _ACTIVE_TEMPLATES.templates[name]
        raise TemplateDoesNotExist("'{}' has no active template.".format(name))
    try:
        return Template.objects.get(name=name, is_active=True)
    except Template.DoesNotExist:
        raise TemplateDoesNotExist("'{}' is neither active, nor found in database.".format(name))


class override_template(ContextDecorator):
    """Context manager/decorator to force active template."""

    def __init__(self, template: Template):
        self.template = template
        self._old_templates = None

    def __enter__(self):
        self._old_templates = getattr(_ACTIVE_TEMPLATES, 'templates', None)

        active_templates = {}
        template = self.template
        while template is not None:
            active_templates[template.name] = template
            template = template.parent
        _ACTIVE_TEMPLATES.templates = active_templates

    def __exit__(self, type, value, traceback):
        if self._old_templates is None:
            del _ACTIVE_TEMPLATES.templates
        else:
            _ACTIVE_TEMPLATES.templates = self._old_templates


class TemplateLoader(Loader):
    """Template loader which returns active templates from `Template` model."""

    def get_template_sources(self, template_name: str) -> Origin:
        """Return template origin."""
        yield Origin(name=template_name, template_name=template_name, loader=self)

    def get_contents(self, origin: Origin) -> str:
        """Return template contents or raise an exception.

        Raises:
            TemplateDoesNotExist: If template was not found.
        """
        template_file = get_active_template(origin.name).content
        try:
            return template_file.read().decode(encoding=self.engine.file_charset)
        finally:
            template_file.close()
